--------------------------------------------------------
-- Timing Generator for CAMtoVGA display and uCom I/O --
--------------------------------------------------------
LIBRARY ieee;
USE ieee.std_logic_1164.all;
use IEEE.STD_LOGIC_ARITH.ALL;
use IEEE.STD_LOGIC_UNSIGNED.ALL;

ENTITY timing_gen IS
	PORT	(
		sclk	: IN std_logic;
		reset_n	: IN std_logic;
		CVSYNC	: IN std_logic;
		CHSYNC	: IN std_logic;
		PCLK	: IN std_logic;
		MCLK	: OUT std_logic;
		CapStart	: IN std_logic;
		CapEnd	: OUT std_logic;
		CapBusy	: OUT std_logic;
		CamWriteAddr	: OUT std_logic_vector(19 downto 0);
		CamWriteWin	: OUT std_logic;
		CVSYNC_edge	: OUT std_logic;
		nLBLANK : OUT std_logic;
		LCLK : OUT std_logic;
		LVSYNC : OUT std_logic;
		LHSYNC : OUT std_logic;
		pclkcnt	:	out	std_logic_vector(1 downto 0)
		);
	END timing_gen;

ARCHITECTURE structure OF timing_gen IS

signal CapBusy_sig	: std_logic;
signal CapEnd_sig	: std_logic;
signal CamLineCount	: std_logic_vector(9 downto 0);
signal CamSegCount	: std_logic_vector(10 downto 0);
signal CVSYNC_dly0	: std_logic;
signal CVSYNC_dly1	: std_logic;
signal CVSYNC_edge_sig	: std_logic;
signal CHSYNC_dly0	: std_logic;
signal CHSYNC_dly1	: std_logic;
signal CHSYNC_edge	: std_logic;
signal AddOut_H	: std_logic_vector(19 downto 0);
signal Offset_H	: std_logic_vector(19 downto 0);
signal CamActiveWin_V : std_logic;
signal CamActiveWin_H : std_logic;
signal MCLK_sig	: std_logic;
signal CamWriteWin_raw : std_logic;
signal CamReadWin_raw : std_logic;
signal PCLK_sig : std_logic;
signal LVSYNC_sig : std_logic;
signal LHSYNC_sig : std_logic;
--signal VgaReadWin_sig : std_logic;

-------------------------------------------------------------- 
BEGIN
--------------------------------------------------------------
pclkcnt <= camsegcount(1 downto 0);		--mu0522
 
gen_CVSYNC_dly : process( reset_n, PCLK )  
begin 
	if( reset_n = '0' ) then
		CVSYNC_dly1 <= '0';
		CVSYNC_dly0 <= '0';
	elsif( PCLK'event and PCLK = '1' ) then
		CVSYNC_dly1 <= CVSYNC_dly0; 
		CVSYNC_dly0 <= CVSYNC; 
	end if;  
end process;
 
CVSYNC_edge_sig <= CVSYNC_dly0 and not CVSYNC_dly1;
CVSYNC_edge <= CVSYNC_edge_sig;

-------------------------------------------------------------- 
gen_CHSYNC_dly : process( reset_n, PCLK )  
begin 
	if( reset_n = '0' ) then
		CHSYNC_dly1 <= '0';
		CHSYNC_dly0 <= '0';
	elsif( PCLK'event and PCLK = '1' ) then
		CHSYNC_dly1 <= CHSYNC_dly0; 
		CHSYNC_dly0 <= CHSYNC; 
	end if;  
end process;

-------------------------------------------------------------- 
CHSYNC_edge <= CHSYNC_dly0 and not CHSYNC_dly1;

-------------------------------------------------------------- 

gen_CamLineCount : process( reset_n, PCLK )  
begin 
	if( reset_n = '0' ) then
		CamLineCount<= (others => '0');	--'1');
	elsif( PCLK'event and PCLK = '1' ) then
		if(CVSYNC_edge_sig = '1') then 
			CamLineCount<= (others => '0');	--'1');
		elsif(CHSYNC_edge = '1') then 
			CamLineCount<= CamLineCount+ 1;
		end if; 
	end if;  
end process;

-------------------------------------------------------------- 
gen_CamSegCount : process( reset_n, PCLK )  
begin 
	if( reset_n = '0' ) then
		CamSegCount<= (others => '0');
	elsif( PCLK'event and PCLK = '1' ) then
		if(CHSYNC_edge = '1' or CamSegCount = 784*2) then
			CamSegCount<= (others => '0');
		else
			CamSegCount<= CamSegCount+ 1;
		end if; 
	end if;  
end process;

-------------------------------------------------------------- 
CamActiveWin_V <= '1' when CamLineCount >= 0 and CamLineCount < 480 else '0';
CamActiveWin_H <= '1' when CamSegCount(10 downto 1) > 0 and CamSegCount(10 downto 1) <= 640 else '0';

-------------------------------------------------------------- 
CamWriteWin <= CamActiveWin_V and CamActiveWin_H and CapBusy_sig;		--mu0523

----------------------------------------------------
CapEnd_sig <= '1' when CapBusy_sig = '1'
and CamLineCount = 479
and CamSegCount(10 downto 1) = 640		--670		mu0523
else '0';
	
CapEnd <= CapEnd_sig;

-------------------------------------------------------------- 
gen_CapBusy : process( reset_n, PCLK )  
begin 
	if( reset_n = '0' ) then
		CapBusy_sig <= '0';
	elsif( PCLK'event and PCLK = '1' ) then
	   if( CapStart = '1' and CVSYNC_edge_sig = '1' ) then 
         CapBusy_sig <= '1';
      elsif( CapEnd_sig = '1' ) then 
         CapBusy_sig <= '0';
      end if; 
	end if;  
end process;

----------------------------------------------------
CapBusy <= CapBusy_sig;
   
----------------------------------------------------
gen_Offset_H : process( reset_n, PCLK )
begin
  if( reset_n = '0' ) then
    Offset_H <= (others => '0');
  elsif( PCLK'event and PCLK = '1' ) then
    if( CVSYNC_edge_sig = '1' ) then 
      Offset_H <= (others => '0');
    elsif( CHSYNC_edge = '1' and CamActiveWin_V = '1' ) then
      Offset_H <= Offset_H + 640;
    end if;
  end if;
end process;

CamWriteAddr <= Offset_H + CamSegCount(10 downto 1);

-------------------------------------------------------------- 
gen_MCLK_sig : process( reset_n, sclk )  
begin 
	if( reset_n = '0' ) then
		MCLK_sig <= '0';
	elsif( sclk'event and sclk = '1' ) then
			MCLK_sig <= not MCLK_sig;
	end if;  
end process;
 
MCLK <= MCLK_sig;
   
--------------------------------------------------------------
LHSYNC_sig <= '1' when CamSegCount(10 downto 1) >= 656 and CamSegCount(10 downto 1) <736 else '0';		--mu0516

LVSYNC_sig <= '1' when CamLineCount >= 2 else '0';

nLBLANK <= CamActiveWin_V and CamActiveWin_H;
-------------------------------------------------------------- 
LCLK <= MCLK_sig;
LHSYNC <= LHSYNC_sig;
LVSYNC <= LVSYNC_sig;
-------------------------------------------------------------- 

END structure;
